==> xfxwidgets-1.9.5.tar.gz: lib/XfxUpdater.dart <==
part of xfxwidgets;

class XfxUpdater extends StatefulWidget {
  XfxUpdater({
    Key? key,
    required this.xMainAPP,
    required this.nextPage_WithOutUpdate,
    required this.nameAPK_APP,
    this.title_Text = "",
    this.title_Color,
    this.title_Style,
    this.onCancel,
  }) : super(key: key);
  final String? title_Text;
  final Color? title_Color;
  final TextStyle? title_Style;
  final XApp? xMainAPP;
  final String nameAPK_APP;
  final String? nextPage_WithOutUpdate;
  final Function()? onCancel;

  @override
  _XfxUpdaterState createState() => _XfxUpdaterState();
}

class _XfxUpdaterState extends State<XfxUpdater> {
  bool XUpdateEnabled = true;
  String latestVers = "";
  late XAzureStorage xb;

  Widget? page;
  @override
  void initState() {
    super.initState();
    if (widget.xMainAPP!.blob_ConnString_XUpdater != null && widget.xMainAPP!.blob_ConnString_XUpdater != "") {
      xb = XAzureStorage.parse(widget.xMainAPP!.blob_ConnString_XUpdater!);
    }
    XCheckForUpdates();
  }

  Future<Widget?> XCheckForUpdates() async {
    if (XUpdateEnabled.not()) return null; //david 05/10/2022 09:58:53

    if (widget.xMainAPP!.blob_ConnString_XUpdater! != "") {
      var appUpdatesHistoryFile = await xb.getBlob("xupdater/${widget.xMainAPP!.blob_nameAPP}.txt");
      if (appUpdatesHistoryFile.statusCode == 200) {
        var x = await appUpdatesHistoryFile.stream.bytesToString();
        if (x != "") {
          latestVers = x.GetLine(x, 0).split('\t')[0];
        }
        // dobbiamo convertire la string in numeri per fare un confronto
        var partsOflatestVers = latestVers.split('.');
        if (latestVers.contains("+")) {
          partsOflatestVers[2] = partsOflatestVers[2].split("+")[0];
        }
        // 3.0.3 => 400000+1000+30
        // 3.0.4 => 400000+1000+40 =>
        var valOfLastVers = (double.parse(partsOflatestVers[0]) + 1) * 100000 + (double.parse(partsOflatestVers[1]) + 1) * 1000 + double.parse(partsOflatestVers[2]) * 10;
        var partsOfapp_Version = widget.xMainAPP!.app_Version.split('.');
        if (partsOfapp_Version.contains("+")) {
          partsOfapp_Version[2] = partsOfapp_Version[2].split("+")[0];
        }

        var app_Version = (double.parse(partsOfapp_Version[0]) + 1) * 100000 + (double.parse(partsOfapp_Version[1]) + 1) * 1000 + double.parse(partsOfapp_Version[2]) * 10;
        //se l'ultima versione è diversa da quella dell'app allora scarico l'apk dal blob
        if (valOfLastVers > app_Version) {
          setState(() {
            page = _buildUI();
          });
          // update_APP();
        } else {
          var x = latestVers.split('+');
          if (x.length > 1) {
            var ver = x[1];
            if ((int.parse(ver) <= int.parse(widget.xMainAPP!.app_BuildNumber))) {
              Navigator.pushNamed(context, widget.nextPage_WithOutUpdate!);
              return null;
            } else {
              setState(() {
                page = _buildUI();
              });
            }
          } else {
            Navigator.pushNamed(context, widget.nextPage_WithOutUpdate!);
            return null;
          }
        }
        if (page != null) {
          return page;
        } else {
          Navigator.pushNamed(context, widget.nextPage_WithOutUpdate!);
          return null;
        }
      } else {
        Navigator.pushNamed(context, widget.nextPage_WithOutUpdate!);
        return null;
      }
    } else {
      Navigator.pushNamed(context, widget.nextPage_WithOutUpdate!);
      return null;
    }
  }

  update_APP() async {
    ///Prima
    // var downDirs = Directory("/storage/emulated/0/Android/data/it.spinwave.tknizerapp/files");
    // var downDirs = await getApplicationDocumentsDirectory();
    var downDirs = await getExternalStorageDirectory();

    ///Dopo
    // var downDirs = Directory('/storage/emulated/0/Download/${widget.xMainAPP!.app_Name.replaceAll(" ", "")}');
    if (downDirs!.existsSync().not()) {
      downDirs.create();
    } else {}
    var baseAppDir = downDirs;
    var outPath = baseAppDir.path; // "";
    // var msg = "Ho trovato una nuova versione: ${latestVers}, la sto scaricando ... attendi prego.";
    print("${widget.xMainAPP!.blob_nameAPP} ${latestVers} DownloadToFile ... in ${outPath}");

    //Scarico l'apk da blob
    var x = await xb.getBlob("${widget.xMainAPP!.blob_nameAPP}/${widget.xMainAPP!.blob_nameAPP}-${latestVers}.apk");
    print(x.statusCode);

    var d = DateTime.now();
    var UIfile = await XLibStorage.CreateFile(widget.nameAPK_APP, extensionFile: "apk", path: outPath);
    print(UIfile!.path); //baseDir.path + "/" + "MELapp.apk"); //${sex.app_Version}
    print("it.spinwave.${widget.xMainAPP!.blob_nameAPP!.toLowerCase()}.fileprovider");
    var buffer = await x.stream.toBytes();
    await UIfile.writeAsBytes(buffer);
    var e = DateTime.now();
    print(e.difference(d).inSeconds.toString());
    print(UIfile.path);
    await OpenFile.open(UIfile.path);
    // var u = Uri(
    //   scheme: "content",
    //   host: "it.spinwave.${widget.xMainAPP!.blob_nameAPP!.toLowerCase()}.fileprovider",
    //   pathSegments: ["external_files", "${widget.nameAPK_APP}.apk"],
    //   // path: UIfile.path,
    // );

    // // Create an AndroidIntent to open the APK file
    // final intent = await AndroidIntent(
    //   action: 'action_view',
    //   flags: [Flag.FLAG_GRANT_READ_URI_PERMISSION, Flag.FLAG_ACTIVITY_NEW_TASK],
    //   data: u.toString(),
    //   type: 'application/vnd.android.package-archive',
    // );
    // await intent.launch();
    exit(0);
  }

  _buildUI() {
    bool isBusy = false;
    return Scaffold(
      backgroundColor: XColors.backgroundDark,
      body: StatefulBuilder(builder: (context, setState) {
        return Column(
          children: [
            Expanded(
                flex: 6,
                child: Container(
                    alignment: Alignment.topCenter,
                    margin: EdgeInsets.symmetric(horizontal: 15),
                    child: SingleChildScrollView(
                        child: Column(mainAxisSize: MainAxisSize.min, children: [
                      SizedBox(height: 60),
                      widget.title_Text != null
                          ? Row(children: [
                              Expanded(
                                  child: XContainerWithLabel(
                                widget.title_Text,
                                color: XColors.backgroundDark,
                                textStyle: widget.title_Style ?? XStyles.xStyTextForLargeTitle(textColor: widget.title_Color ?? XColors.foregroundLight),
                              ))
                            ])
                          : Container(),
                      SizedBox(height: 3),
                      Row(children: [
                        Expanded(
                            child: Row(children: [
                          Text("Vers. attuale", style: XStyles.xStyTextForDescr()),
                          Text(" ${widget.xMainAPP!.app_Version}", style: XStyles.xStyTextForSubLabel(textColor: Colors.yellow)),
                        ])),
                        SizedBox(width: 10),
                        Expanded(
                            child: Row(children: [
                          Text("Vers. in CLOUD", style: XStyles.xStyTextForDescr()),
                          Text(" ${latestVers}", style: XStyles.xStyTextForSubLabel(textColor: Colors.green)),
                        ])),
                      ]),
                      SizedBox(height: 20),
                      Container(child: XStatefulWidgetBase.LoaderIndicator(isBusy) ?? Container()),
                      SizedBox(height: 5),
                    ])))),
            Expanded(
              child: isBusy.not()
                  ? Container(
                      child: Row(children: [
                      Expanded(
                          child: XBtnbase(
                        height: 60,
                        label: "Continua \nsenza aggiornare",
                        onPressed: widget.onCancel,
                        label_Color: Colors.yellow,
                      )),
                      Expanded(
                          child: XBtnbase(
                        height: 60,
                        label: "Aggiorna",
                        onPressed: () async {
                          setState(() {
                            isBusy = true;
                          });
                          await update_APP();
                        },
                        label_Color: Colors.greenAccent,
                      )),
                    ]))
                  : Container(),
            )
          ],
        );
      }),
    );
  }

  @override
  Widget build(BuildContext context) {
    return page ?? Container();
  }
}

==> xfxwidgets-1.9.99.tar.gz: lib/XfxUpdater.dart <==
part of xfxwidgets;

class XfxUpdater extends StatefulWidget {
  XfxUpdater({
    Key? key,
    required this.xMainAPP,
    required this.nextPage_WithOutUpdate,
    required this.nameAPK_APP,
    this.title_Text = "",
    this.title_Color,
    this.title_Style,
    this.onCancel,
  }) : super(key: key);
  final String? title_Text;
  final Color? title_Color;
  final TextStyle? title_Style;
  final XApp? xMainAPP;
  final String nameAPK_APP;
  final String? nextPage_WithOutUpdate;
  final Function()? onCancel;

  @override
  _XfxUpdaterState createState() => _XfxUpdaterState();
}

class _XfxUpdaterState extends State<XfxUpdater> {
  bool XUpdateEnabled = true;
  String latestVers = "";
  late XAzureStorage xb;

  Widget? page;
  @override
  void initState() {
    super.initState();
    if (widget.xMainAPP!.blob_ConnString_XUpdater != null && widget.xMainAPP!.blob_ConnString_XUpdater != "") {
      xb = XAzureStorage.parse(widget.xMainAPP!.blob_ConnString_XUpdater!);
    }
    XCheckForUpdates();
  }

  Future<Widget?> XCheckForUpdates() async {
    if (XUpdateEnabled.not()) return null; //david 05/10/2022 09:58:53

    if (widget.xMainAPP!.blob_ConnString_XUpdater! != "") {
      var appUpdatesHistoryFile = await xb.getBlob("xupdater/${widget.xMainAPP!.blob_nameAPP}.txt");
      if (appUpdatesHistoryFile.statusCode == 200) {
        var x = await appUpdatesHistoryFile.stream.bytesToString();
        if (x != "") {
          latestVers = x.GetLine(x, 0).split('\t')[0];
        }
        // dobbiamo convertire la string in numeri per fare un confronto
        var partsOflatestVers = latestVers.split('.');
        if (latestVers.contains("+")) {
          partsOflatestVers[2] = partsOflatestVers[2].split("+")[0];
        }
        // 3.0.3 => 400000+1000+30
        // 3.0.4 => 400000+1000+40 =>
        var valOfLastVers = (double.parse(partsOflatestVers[0]) + 1) * 100000 + (double.parse(partsOflatestVers[1]) + 1) * 1000 + double.parse(partsOflatestVers[2]) * 10;
        var partsOfapp_Version = widget.xMainAPP!.app_Version.split('.');
        if (partsOfapp_Version.contains("+")) {
          partsOfapp_Version[2] = partsOfapp_Version[2].split("+")[0];
        }

        var app_Version = (double.parse(partsOfapp_Version[0]) + 1) * 100000 + (double.parse(partsOfapp_Version[1]) + 1) * 1000 + double.parse(partsOfapp_Version[2]) * 10;
        //se l'ultima versione è diversa da quella dell'app allora scarico l'apk dal blob
        if (valOfLastVers > app_Version) {
          setState(() {
            page = _buildUI();
          });
          // update_APP();
        } else {
          var x = latestVers.split('+');
          if (x.length > 1) {
            var ver = x[1];
            if ((int.parse(ver) <= int.parse(widget.xMainAPP!.app_BuildNumber))) {
              Navigator.pushNamed(context, widget.nextPage_WithOutUpdate!);
              return null;
            } else {
              setState(() {
                page = _buildUI();
              });
            }
          } else {
            Navigator.pushNamed(context, widget.nextPage_WithOutUpdate!);
            return null;
          }
        }
        if (page != null) {
          return page;
        } else {
          Navigator.pushNamed(context, widget.nextPage_WithOutUpdate!);
          return null;
        }
      } else {
        Navigator.pushNamed(context, widget.nextPage_WithOutUpdate!);
        return null;
      }
    } else {
      Navigator.pushNamed(context, widget.nextPage_WithOutUpdate!);
      return null;
    }
  }

  update_APP() async {
    ///Prima
    // var downDirs = Directory("/storage/emulated/0/Android/data/it.spinwave.tknizerapp/files");
    // var downDirs = await getApplicationDocumentsDirectory();
    var downDirs = await getExternalStorageDirectory();

    ///Dopo
    // var downDirs = Directory('/storage/emulated/0/Download/${widget.xMainAPP!.app_Name.replaceAll(" ", "")}');
    if (downDirs!.existsSync().not()) {
      downDirs.create();
    } else {}
    var baseAppDir = downDirs;
    var outPath = baseAppDir.path; // "";
    // var msg = "Ho trovato una nuova versione: ${latestVers}, la sto scaricando ... attendi prego.";
    print("${widget.xMainAPP!.blob_nameAPP} ${latestVers} DownloadToFile ... in ${outPath}");

    //Scarico l'apk da blob
    var x = await xb.getBlob("${widget.xMainAPP!.blob_nameAPP}/${widget.xMainAPP!.blob_nameAPP}-${latestVers}.apk");
    print(x.statusCode);

    var d = DateTime.now();
    var UIfile = await XLibStorage.CreateFile(widget.nameAPK_APP, extensionFile: "apk", path: outPath);
    print(UIfile!.path); //baseDir.path + "/" + "MELapp.apk"); //${sex.app_Version}
    print("it.spinwave.${widget.xMainAPP!.blob_nameAPP!.toLowerCase()}.fileprovider");
    var buffer = await x.stream.toBytes();
    await UIfile.writeAsBytes(buffer);
    var e = DateTime.now();
    print(e.difference(d).inSeconds.toString());
    print(UIfile.path);
    await OpenFile.open(UIfile.path);
    // var u = Uri(
    //   scheme: "content",
    //   host: "it.spinwave.${widget.xMainAPP!.blob_nameAPP!.toLowerCase()}.fileprovider",
    //   pathSegments: ["external_files", "${widget.nameAPK_APP}.apk"],
    //   // path: UIfile.path,
    // );

    // // Create an AndroidIntent to open the APK file
    // final intent = await AndroidIntent(
    //   action: 'action_view',
    //   flags: [Flag.FLAG_GRANT_READ_URI_PERMISSION, Flag.FLAG_ACTIVITY_NEW_TASK],
    //   data: u.toString(),
    //   type: 'application/vnd.android.package-archive',
    // );
    // await intent.launch();
    exit(0);
  }

  _buildUI() {
    bool isBusy = false;
    return Scaffold(
      backgroundColor: XColors.backgroundDark,
      body: StatefulBuilder(builder: (context, setState) {
        return Column(
          children: [
            Expanded(
                flex: 6,
                child: Container(
                    alignment: Alignment.topCenter,
                    margin: EdgeInsets.symmetric(horizontal: 15),
                    child: SingleChildScrollView(
                        child: Column(mainAxisSize: MainAxisSize.min, children: [
                      SizedBox(height: 60),
                      widget.title_Text != null
                          ? Row(children: [
                              Expanded(
                                  child: XContainerWithLabel(
                                widget.title_Text,
                                color: XColors.backgroundDark,
                                textStyle: widget.title_Style ?? XStyles.xStyTextForLargeTitle(textColor: widget.title_Color ?? XColors.foregroundLight),
                              ))
                            ])
                          : Container(),
                      SizedBox(height: 3),
                      Row(children: [
                        Expanded(
                            child: Row(children: [
                          Text("Vers. attuale", style: XStyles.xStyTextForDescr()),
                          Text(" ${widget.xMainAPP!.app_Version}+${widget.xMainAPP!.app_BuildNumber}", style: XStyles.xStyTextForSubLabel(textColor: Colors.yellow)),
                        ])),
                        SizedBox(width: 10),
                        Expanded(
                            child: Row(children: [
                          Text("Vers. in CLOUD", style: XStyles.xStyTextForDescr()),
                          Text(" ${latestVers}", style: XStyles.xStyTextForSubLabel(textColor: Colors.green)),
                        ])),
                      ]),
                      SizedBox(height: 20),
                      Container(child: XStatefulWidgetBase.LoaderIndicator(isBusy) ?? Container()),
                      SizedBox(height: 5),
                    ])))),
            Expanded(
              child: isBusy.not()
                  ? Container(
                      child: Row(children: [
                      Expanded(
                          child: XBtnbase(
                        height: 60,
                        label: "Continua \nsenza aggiornare",
                        onPressed: widget.onCancel,
                        label_Color: Colors.yellow,
                      )),
                      Expanded(
                          child: XBtnbase(
                        height: 60,
                        label: "Aggiorna",
                        onPressed: () async {
                          setState(() {
                            isBusy = true;
                          });
                          await update_APP();
                        },
                        label_Color: Colors.greenAccent,
                      )),
                    ]))
                  : Container(),
            )
          ],
        );
      }),
    );
  }

  @override
  Widget build(BuildContext context) {
    return page ?? Container();
  }
}
